@import 'core/css/variables';

.wordpress {
    h1 a, h1 a:hover, h1 a:focus, h1 a:visited,
    h2 a, h2 a:hover, h2 a:focus, h2 a:visited {
        text-decoration: none;
    }

    .post {
        margin-bottom: 40px;

        .post-date,
        .post-footer,
        .post-nav {
            color: #999;
        }
        .post-nav {
            margin-top: 10px;
        }
    }
    .comments {
        margin-top: 20px;
        margin-bottom: 20px;

        .comment {
            border-left: 3px solid rgba(0 ,0 ,0 ,0.2);;
            padding-left: 10px;

            margin-top: 20px;
            margin-bottom: 20px;
        }
        .comment-form {
            margin: 0;
        }
        #respond {
            margin-top: 20px;
            margin-bottom: 20px;

            h3 {
                font-size: 20px;
                line-height: 30px;
            }
        }
    }
    .news {
        margin-top: 40px;

        .news-entry-date {
            text-overflow: ellipsis;
        }
    }
    .wp-caption {
        padding: 0 5px;
        border-radius: 5px;
    }
    .wp-caption-text {
        font-size: small;
        margin-top: 5px;
    }
    .alignright {
        margin-left: 20px;
        margin-bottom: 20px;
        float: right;
    }
    .alignleft {
        margin-right: 20px;
        margin-bottom: 20px;
        float: left;
    }
    .alignright,
    .alignleft {
        @media screen and (max-width: $screen-xs-max) {
            float: none;
            margin: 0 auto;
            max-width: 100%;
        }
    }
    .aligncenter {
        display: block;
        margin-left: auto;
        margin-right: auto;
        margin-top: 20px;
        margin-bottom: 20px;
    }
    .news-entry {
        margin-bottom: 10px;
    }
    .news-entry-date {
        width: 180px;
        display: inline-block;
    }

    .sidebar {
        h3 {
            margin-bottom: 10px;
            font-size: 22px;
        }
        form, input {
            margin: 0;
        }
        ul {
            margin-left: 0px;
            padding-left: 0px;

            li {
                margin-bottom: 20px;

                ul li {
                    margin-bottom: 0;
                }
            }
        }
        li {
            list-style: none;
        }
        .children {
            li {
                margin-left: 10px;

                &:before {
                    content: "- ";
                }
            }
        }
    }
    .post-video {
        float: left;

        video, img {
            width: 320px;
        }
    }
    .post-edit-link i {
        font-size: 16px;
    }
}
