{% extends 'core/page.html' %}
{% load i18n %}
{% load core_tags %}

{% block page %}

    <h1>{% trans 'Registration' %}</h1>

    <p>{% trans 'The registration is currently closed.' %}

{% endblock %}
