# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-23 13:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_meetings', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='meeting',
            name='registration_done_message',
            field=models.TextField(blank=True, help_text='Message on the page displayed after registration', null=True, verbose_name='Registration done message'),
        ),
    ]
