# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-05 13:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_archive', '0002_collection'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='archivejob',
            name='file_path',
        ),
    ]
