{% load i18n %}

<div class="modal" id="daiquiri-table-modal" tabindex="-1">
    <div class="modal-dialog modal-lg">
        <div class="modal-content">
            <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal">
                    <span aria-hidden="true">&times;</span>
                </button>
                <h4 class="modal-title">
                    {$ service.table.modal.title $}
                </h4>
            </div>

            <div class="modal-body">
                <pre ng-show="service.table.modal.pre">{$ service.table.modal.pre $}</pre>
                <img class="center-block" ng-show="service.table.modal.src" ng-src="{$ service.table.modal.src $}" />
            </div>

            <div class="modal-footer">
                <div class="pull-left">
                    <button type="button" class="btn btn-default" ng-click="service.table.modal_left()">
                        {% trans 'Previous column' %}
                    </button>
                    <button type="button" class="btn btn-default" ng-click="service.table.modal_right()">
                        {% trans 'Next column' %}
                    </button>
                </div>
                <div>
                    <button type="button" class="btn btn-default" ng-click="service.table.modal_up()">
                        {% trans 'Previous row' %}
                    </button>
                    <button type="button" class="btn btn-default" ng-click="service.table.modal_down()">
                        {% trans 'Next row' %}
                    </button>
                </div>
            </div>
        </div>
    </div>
</div>
