{% extends 'core/page.html' %}
{% load i18n %}

{% block page %}

    <h1>{% trans "Contact Form" %}</h1>

    <p>
        {% trans "Please enter your contact details and your message." %}
    </p>

    <form method="post" action="{% url 'contact:contact' %}" novalidate>
        {% csrf_token %}
        <input type="hidden" name="next" value="{{ next }}" />

        {% include 'core/partials/form_fields.html' %}

        <input type="submit" value="{% trans 'Send message' %}" class="btn {{ submit_danger|yesno:'btn-danger,btn-primary' }}" />
        <input type="submit" name="cancel" value="{% trans 'Cancel' %}" class="btn btn-default" />
    </form>

{% endblock %}
